#ifndef COMFIO_H
#define COMFIO_H

using namespace std;



#include <sstream>
#include <string>
#include <cstdio>
#include <cstring>
#include <cstdarg>
#include <vector>
#include <climits>
#include "functor.h"


/*! class provides cout/cerr-style and printf-style input operations for derived classes */

class ComfIO
{
	private:
		TFunctor*		eots_call_ptr;
		char*			comfiobuf;
		/*!
		string containing the sequence of characters evaluated as end-of-line
		when doing line-read or line-write operations
		*/
		string*			eots;
		ssize_t 		max_msg;
		ssize_t 		has_eots();
		ssize_t			bytes_to_send;
		stringstream	comfiostream;
		ssize_t			vscanf(const char *format, va_list ap);
		ssize_t			getBytesToSend() {return bytes_to_send;}
		pthread_mutex_t	amutex;
		bool 			atomic;
		bool			cons_debug;
		const char**	ioctrls_chars;
		friend class	Io;
		void			checkBase(int base, const char* cptr);
	protected:
		bool			is_eots(const char* s);
		bool			is_eots(string& s);
		ssize_t 		fwriten(const char* format, va_list ap);
		void			clearBuffer();
		/*!
			returns: 
				- pointer to input buffer
		*/
		stringstream*	buffer() {return &comfiostream;}
		virtual void	setActOnEOTS(TFunctor* ptr);
		virtual void    setEOTS(const char* eot);
		virtual void    setEOTS(string& eot);
		ssize_t 	count_chars(const char*s, char c);
		
	public:
		ComfIO(ssize_t bufsize = 1024);
		virtual 	~ComfIO();
		void		writeConsole(const char* prefix, const char* str, size_t len = 0);
		bool		lockMutex(const char* where = 0);
		bool		unlockMutex(const char* where = 0);
		void		atomicIO(bool on);
		/*!
			returns: 
				- atomic IO setting
		*/
		bool		atomicIO() {return atomic;}
		void		consoleDebug(bool on);
        enum		ctrls_chars { NUL, SOH, STX, ETX, EOT, ENQ, ACK, BEL, BS, HT,
					   LF, VT, FF, CR, SO, SI, DLE, DC1, DC2, DC3,
					   DC4, NAK, SYN, ETB, CAN, EM, SUB, ESC, FS, GS,
				       RS, US, DEL=127 };
		/*!

			returns: 

				- true: console debugging is turned on
				- false:  console debugging is turned off
			
		*/
		bool		consoleDebug()	{return cons_debug;}


		ssize_t 		getIOBufSize();
		bool			setIOBufSize(ssize_t newsize);
		ssize_t  		write(const void *buf, size_t nbytes);
		ssize_t  		printf(const char* format, ...);
		const char 		*getEOTS();
		static ssize_t 	tokenize(string& in, vector<string>& out, bool zero = true, const char* delims = 0);
		TFunctor*		getActOnEOTS();
		ComfIO& operator<<(char c);
		ComfIO& operator<<(unsigned char c);
		ComfIO& operator<<(signed char sc);
		ComfIO& operator<<(const char *cptr);
		ComfIO& operator<<(const unsigned char *ucptr);
		ComfIO& operator<<(const signed char *scptr);
		ComfIO& operator<<(string& str);
		ComfIO& operator<<(string* str);
		ComfIO& operator<<(short s);
		ComfIO& operator<<(unsigned short s);
		ComfIO& operator<<(int i);
		ComfIO& operator<<(unsigned int ui);
		ComfIO& operator<<(long l);
		ComfIO& operator<<(unsigned long l);
		ComfIO& operator<<(float f);
		ComfIO& operator<<(double d);
		ComfIO& operator<<(long double d);
		ComfIO& operator<<(ostream& ( *pf )(ostream&));
		ComfIO& operator<<(ios_base& ( *pf )(ios_base&));
		ComfIO& operator<<(ios& ( *pf )(ios&));
		
		/*ComfIO& operator>>(string& str);
		ComfIO& operator>>(int i);
		ComfIO& operator>>(double d);*/

		double  		toDouble(const char *cptr);
		/*!
	 		convert str to double and return it

			Can throw:

			- IpcConvertException
	
		*/
		double			toDouble(string& str) {return toDouble(str.c_str());}
		float  			toFloat(const char *cptr);
		/*!
	 		convert str to float and return it

			Can throw:

			- IpcConvertException
	
		*/
		float			toFloat(string& str) {return toFloat(str.c_str());}
		unsigned long	toUlong(const char *cptr, int base = 0);
		/*!
	 		convert str to unsigned long using base to and return it

			Can throw:

			- IpcConvertException
	
		*/
		unsigned long	toUlong(string& str, int base = 0) {return toUlong(str.c_str(), base);}
		long			toLong(const char *cptr, int base = 0);
		/*!
	 		convert str to long using base to and return it

			Can throw:

			- IpcConvertException
	
		*/
		long			toLong(string& str, int base = 0) {return toLong(str.c_str(), base);}
		unsigned long long	toUlongLong(const char *cptr, int base = 0);
		/*!
	 		convert str to unsigned long long using base to and return it

			Can throw:

			- IpcConvertException
	
		*/
		unsigned long long	toUlongLong(string& str, int base = 0) {return toUlongLong(str.c_str(), base);}
		long long		toLongLong(const char *cptr, int base = 0);
		/*!
	 		convert str to long long using base to and return it

			Can throw:

			- IpcConvertException
	
		*/
		long long		toLongLong(string& str, int base = 0) {return toLongLong(str.c_str(), base);}
};


#endif

