#ifndef FUNCTOR_H
#define FUNCTOR_H
/*! pure virtual base class for implementing functor\n 
(cf.
<A HREF="http://www.newty.de/fpt/functor.html" target="_blank">Function pointer tutorial</A>)
*/

class TFunctor
{
   public:
		/*! virtual destructor, implemented to calm down compiler */
		virtual ~TFunctor() {}
		/*! virtual () operator   */     
		virtual ssize_t operator()() = 0;
		/*! overloaded virtual ()(void* buf, size_t len) operator   */
		virtual void	operator()(void* buf, size_t len)		 = 0;
};

/*! Template functor class to implement callback function pointers  */

template <class TClass> class TSpecificFunctor : public TFunctor
{
	private:
		ssize_t (TClass::*fpt1)();   // pointer to member function
		void	(TClass::*fpt2)(void* buf, size_t len);
		TClass* pt2Object;          // pointer to object
	public:
	  /*! constructor for TSpecificFunctor
	
	      in - _pt2Object: pointer to object

		  in - _fpt:       pointer to object method of type ssize_t method()
	 */
	TSpecificFunctor(TClass* _pt2Object, ssize_t (TClass::*_fpt)())
	{
		pt2Object = _pt2Object;
		fpt1 = _fpt;
	};
	/*! constructor for TSpecificFunctor
	
	      in - _pt2Object: pointer to object

		  in - _fpt:       pointer to object method of type void method(void* buf, size_t len)
	 */

	TSpecificFunctor(TClass* _pt2Object, void (TClass::*_fpt)(void* buf, size_t len))
	{
		pt2Object = _pt2Object;
		fpt2 = _fpt;
	};

	 /*! overloaded operator (), calls method passed in constructor */
	virtual ssize_t operator()()
	{
		return (*pt2Object.*fpt1)();
	};
	 /*! overloaded operator (), calls method passed in constructor */
	virtual void operator()(void* buf, size_t len)
	{
		(*pt2Object.*fpt2)(buf, len);
	};
};
#endif


