#include <ipc_exception.h>
#include <iostream>
#include <typeinfo>

/*!
				print exception text(s) to stderr
*/

void IpcException::print() const throw()
{
	cerr << "Caught IpcException: "
		 << what() << endl
		 << "in file:             "
		 << fileName() << endl
		 << "at line:             "
		 << lineNumber() << endl;
}

/*!
				print exception text(s) to stderr
				
				in - fn: the filename of the file the exception was caught
	
				in - ln: the line number in the file where the exception was caught

				do not use this method directly, use macro IPC_PRINTEX instead
*/

void IpcException::print(const char* fn, int ln) const throw()
{
		cerr << classname() << ": "
		<< what() << endl
		<< "thrown in: " << fileName() << ", line " << lineNumber() << endl
		<< "caught in: " << fn << ", line " << ln << endl;
}

/*!
				print exception text(s) to stderr
*/
void IpcScanfException::print() const throw()
{
	cerr << "Caught IpcScanfException: "
		 << what() << endl
		 << "with format:              "
         << formatString() << endl
		 << "and input:                "
         << inputString() << endl
		 << "in file:                  "
		 << fileName() << endl
		 << "at line:                  "
		 << lineNumber() << endl;
}

/*!
				print exception text(s) to stderr
				
				in - fn: the filename of the file the exception was caught
	
				in - ln: the line number in the file where the exception was caught

				do not use this method directly, use macro IPC_PRINTEX instead
*/

void IpcScanfException::print(const char* fn, int ln) const throw()
{
		cerr << classname() << ": "
		<< what() << endl
		<< "thrown in: " << fileName() << ", line " << lineNumber() << endl
		<< "caught in: " << fn << ", line " << ln << endl
		<< "with format:              "
        << formatString() << endl
		<< "and input:                "
        << inputString() << endl;
}


/*!
				print exception text(s) to stderr
*/
void IpcTimeOutException::print() const throw()
{
	cerr << "Caught : IpcTimeOutException"
	<< what() << endl
	<< "in file:                  "
	<< fileName() << endl
	<< "at line:                  "
	<< lineNumber() << endl;
}

/*!
				print exception text(s) to stderr
				
				in - fn: the filename of the file the exception was caught
	
				in - ln: the line number in the file where the exception was caught

				do not use this method directly, use macro IPC_PRINTEX instead
*/

void IpcTimeOutException::print(const char* fn, int ln) const throw()
{
		cerr << classname() << ": "
		<< what() << endl
		<< "thrown in: " << fileName() << ", line " << lineNumber() << endl
		<< "caught in: " << fn << ", line " << ln << endl;
}

/*!
				print exception text(s) to stderr
*/
void IpcConvertException::print() const throw()
{
	cerr << "Caught IpcConvertException: "
		 << what() << endl
		 << "in file:                    "
		 << fileName() << endl
		 << "at line:                    "
		 << lineNumber() << endl
		 << "with input:                 "
		 << input() << endl;
}

/*!
				print exception text(s) to stderr
				
				in - fn: the filename of the file the exception was caught
	
				in - ln: the line number in the file where the exception was caught

				do not use this method directly, use macro IPC_PRINTEX instead
*/

void IpcConvertException::print(const char* fn, int ln) const throw()
{
		cerr << classname() << ": "
		<< what() << endl
		<< "thrown in: " << fileName() << ", line " << lineNumber() << endl
		<< "caught in: " << fn << ", line " << ln << endl
		<< "with input:                 "
		 << input() << endl;
}



