// --- ipc_tcp.h ------------------------------------------------------

#ifndef IPC_TCP_H
#define IPC_TCP_H

#include <iostream>
using namespace std;

extern "C" {
	#include <errno.h>
	#include <unistd.h>	
	#include <netinet/in.h>
	#include <sys/socket.h>     
	#include <sys/time.h>
	#include <sys/un.h>
	#include <netdb.h>
	#include <sys/types.h>
	#include <fcntl.h>
	#include <netinet/tcp.h>
}




#define IPC_TCPNAME 	1
#define IPC_TCPADDR 	2
#define IPC_TCPPORTNAME 3
#define IPC_TCPPORTNUM	4

//-----------------------------------------------------------

class SocketAddr : 	sockaddr, addrinfo 
{
  	public:
		SocketAddr(){}
		SocketAddr(const char* host, const char* port_no);
		SocketAddr(const char* host, unsigned short port_no);
		SocketAddr(const char* sname);
		SocketAddr(const SocketAddr& in);
		~SocketAddr();
	private:
		/*!
 			returns:
				- error state of SocketAddr
				- true : everything ok
				- false : something went wrong
		*/
		bool ok() {return !socket_ok;}
		/*!
 			returns:
				- description of any error on SocketAddr
		*/
		const char* socketErr() {return ::gai_strerror(socket_ok);}
		int socket_ok;
		socklen_t size();
		string sockInfo;
		operator sockaddr* () const;
		unsigned short family() {return ai_family;}
		const char* info(int flag, int sockfd = -1);
		const char* address() {return info(IPC_TCPADDR);}
		const char* name()    {return info(IPC_TCPNAME);}
		const char* pname()	  {return info(IPC_TCPPORTNAME);}
		const char* pnum()	  {return info(IPC_TCPPORTNUM);}
		friend class ConnectSocket;
		sockaddr_un uds_addr;
};	
					
/*!
	Connect socket class

*/
class ConnectSocket
{
	private:
		string			peerInf;
		string			hostAddr;
		string			hostN;
		SocketAddr		storedAddress;
		int				fd;

 	protected:
		ConnectSocket(SocketAddr targetAddr, long timeout);
		~ConnectSocket()	{}

	public:
		/*!
 			returns file descriptor of established connection
		*/
		int	filedescriptor() {return fd;}
		const char* hostAddress();
		const char* hostName();
		const char* hostPname();
		const char* hostPnum();
		const char* peerAddress();
		const char* peerName();
		const char* peerPname();
		const char* peerPnum();
		int			shutdown(int how);
		int			reopen();
        bool 		nodelay();
		bool 		nodelay(bool on);
		bool 		keepalive();
        bool 		keepalive(bool on, int keepalive_interval = 75);

};



/*!
	Client socket class

*/

class TcpSocket : public ConnectSocket
{
	public:
		TcpSocket(const char* host, const char* service, long timeout = 2);
		TcpSocket(const char* host, unsigned short port_no, long timeout = 2);
		~TcpSocket() {}
};

class DomainSocket : public ConnectSocket
{
	public:
		DomainSocket(const char* sname, long timeout = 2);
		~DomainSocket() {}
};

#endif //IPC_TCP_H

